@@ -20,71 +20,17 @@
  * database.c
  */
 #include "global.h"
-#include "stat.h"       /* for start_song_picker() */
 #include "player.h"     /* for player_queue_command() */
 #include "display.h"    /* for draw_info() and draw_screen() */
 #include "play_ogg.h"   /* for ogg_load_meta() */
 #include "play_mp3.h"   /* for mp3_load_meta() */
+#include "play_flac.h"  /* for flac_load_meta() */
+#ifdef EMPEG
+#include "vfdlib.h"     /* for vfdlib_*() */
+#include "version.h"    /* for SQUASH_VERSION */
+#endif
 #include "database.h"
 
-/*
- * This is called by main() to load the DB.  Since nothing else can happen until
- * the database is loaded, it does a few other things, in order to tell the other
- * threads that the database has loaded, and that they can start.
- */
-void *setup_database( void *data ) {
-    /* Grab our locks */
-    squash_lock( display_info.lock );
-    squash_wlock( database_info.lock );
-    squash_lock( song_queue.lock );
-    squash_lock( player_info.lock );
-
-    /* Load the database */
-    load_db_filenames();
-
-    /* Unlock the locks we don't need anymore */
-    squash_unlock( player_info.lock );
-    squash_unlock( song_queue.lock );
-
-    /* Hide the info window, and unhide the spectrum analyzer */
-    display_info.window[ WIN_INFO ].state = WIN_STATE_HIDDEN;
-    display_info.window[ WIN_SPECTRUM ].state = WIN_STATE_NORMAL;
-    display_info.state = SYSTEM_RUNNING;
-
-    /* Tell the specturm analyzer it can run */
-    squash_lock( spectrum_ring.lock );
-    spectrum_ring.active = TRUE;
-    squash_unlock( spectrum_ring.lock );
-
-    /* Update the screen */
-    squash_wunlock( database_info.lock );
-    draw_screen();
-
-    /* Release the database lock */
-    squash_unlock( display_info.lock );
-
-    /* Tell the player to start playing */
-    squash_lock( player_command.lock );
-    player_queue_command( CMD_PLAY );
-    squash_signal( player_command.changed );
-    squash_unlock( player_command.lock );
-
-    /* Tell the playlist_manager() thread to add songs */
-    squash_lock( song_queue.lock );
-    song_queue.wanted_size = config.playlist_manager_playlist_size;
-    squash_signal( song_queue.not_full );
-    squash_unlock( song_queue.lock );
-
-    load_all_meta_data( TYPE_STAT ); /* Load statistics */
-    /* Load the statistics routine (needed for playlist_manager() to call pick_song()) */
-    start_song_picker();
-    squash_signal( database_info.stats_finished );
-
-    load_all_meta_data( TYPE_META ); /* Load info files */
-
-    return (void *)NULL;
-}
-
 /*
  * Loads the database.  Will walk the config.db_paths[ BASENAME_SONG ] directory
  * looking for known music types.  Loading the meta data and stat values are done,
@@ -102,13 +48,18 @@ void load_db_filenames( void ) {
         database_info.stats_loaded = 0;
     }
 
-    /* Check the path to see if it exists */
-    if( stat(config.db_paths[ BASENAME_SONG ], &path_stat) != 0 ) {
-        squash_error( "The path or file '%s' does not exist or I can't open it", config.db_paths[ BASENAME_SONG ] );
-    }
+    /* If we are supposed to use a master list and it already exists */
+    if( config.db_masterlist_path != NULL && stat(config.db_masterlist_path, &path_stat) == 0 ) {
+        load_masterlist();
+    } else {
+        /* Check the path to see if it exists */
+        if( stat(config.db_paths[ BASENAME_SONG ], &path_stat) != 0 ) {
+            squash_error( "The path or file '%s' does not exist or I can't open it", config.db_paths[ BASENAME_SONG ] );
+        }
 
-    /* We need to walk the directory tree and load bunches of files */
-    _walk_filesystem( NULL, _load_file );
+        /* We need to walk the directory tree and load bunches of files */
+        _walk_filesystem( NULL, _load_file );
+    }
 
     squash_log("Songs loaded %d", database_info.song_count);
 
@@ -119,6 +70,77 @@ void load_db_filenames( void ) {
     /* Trim database_info.songs[]'s allocated size */
     database_info.song_count_allocated = database_info.song_count;
     squash_realloc( database_info.songs, database_info.song_count_allocated * sizeof(song_info_t) );
+
+    if( config.db_masterlist_path != NULL ) {
+        save_masterlist();
+    }
+}
+
+void load_masterlist( void ) {
+    FILE *masterlist_file;
+    struct stat file_info;
+    char *file_data;
+    char *cur_data;
+    char *this_line;
+
+    /* Open the file to write out the playlist */
+    if( (masterlist_file = fopen(config.db_masterlist_path, "r")) == NULL ) {
+        squash_log("Couldn't open masterlist file, probably didn't exist");
+        return;
+    }
+
+    if( fstat(fileno(masterlist_file), &file_info) ) {
+        squash_log("Couldn't stat masterlist file");
+        fclose( masterlist_file );
+        return;
+    }
+
+    if( (file_data = (char *)mmap( NULL, file_info.st_size, PROT_READ|PROT_WRITE, MAP_PRIVATE, fileno(masterlist_file), 0)) == MAP_FAILED ) {
+        squash_log("Couldn't mmap masterlist file");
+        fclose( masterlist_file );
+        return;
+    }
+
+    cur_data = file_data;
+    while( cur_data != NULL ) {
+        this_line = strsep( &cur_data, "\n" );
+        /* if not a line, empty, or is a comment */
+        if( !(this_line == NULL || this_line[0] == '\0' || this_line[0] == '#') ) {
+            _load_file( this_line, TRUE );
+        }
+    }
+
+    /* Close the playlist file */
+    fclose( masterlist_file );
+}
+
+void save_masterlist( void ) {
+    FILE *masterlist_file;
+    time_t current_time;
+    int i;
+
+    if( config.db_readonly ) {
+        return;
+    }
+
+    /* Open the file to write out the masterlist */
+    if( (masterlist_file = fopen(config.db_masterlist_path, "w")) == NULL ) {
+        squash_error( "Can't open file \"%s\" for writing", config.db_masterlist_path );
+    }
+
+    /* Write out header */
+    time( &current_time );
+    fprintf( masterlist_file, "# Saved Squash masterlist\n" );
+    fprintf( masterlist_file, "# Auto-Generated on %s\n", ctime(&current_time) );
+
+    /* Write out the masterlist */
+    for( i = 0; i < database_info.song_count; i++ ) {
+        fprintf( masterlist_file, "%s\n", database_info.songs[i].filename );
+    }
+
+    /* Close the masterlist file */
+    fclose( masterlist_file );
+
 }
 
 /*
@@ -228,7 +250,7 @@ db_search_result_t find_matches( char *key, char *keyword ) {
 }
 
 /*
- * Clears everything allocate in a song except for the filename
+ * Clears everything allocated in a song except for the filename
  */
 void clear_song_meta( song_info_t *song ) {
     int j, k;
@@ -300,6 +322,10 @@ void save_song( song_info_t *song ) {
             continue;
         }
 
+        if( db_extensions[i].which_basename == BASENAME_STAT && ! database_info.stats_loaded ) {
+            continue;
+        }
+
         /* Construct a filename to save */
         cur_filename = build_fullfilename( song, db_extensions[i].which_basename );
 
@@ -350,6 +376,7 @@ void save_stat_data( song_info_t *song, FILE *file ) {
     fprintf( file, "play_count=%d\n", song->stat.play_count );
     fprintf( file, "skip_count=%d\n", song->stat.skip_count );
     fprintf( file, "repeat_counter=%d\n", song->stat.repeat_counter );
+    fprintf( file, "manual_rating=%d\n", song->stat.manual_rating );
     fprintf( file, "\n" );
 
     /* Reset the changed flag */
@@ -381,7 +408,6 @@ void save_meta_data( song_info_t *song, FILE *file ) {
 void load_meta_data( song_info_t *song, enum meta_type_e which ) {
     char *filename, *metaname, *dirname;
     char *end;
-    enum song_type_e song_type;
     bool success;
     FILE *meta_file;
     struct stat file_stat;
@@ -391,11 +417,34 @@ void load_meta_data( song_info_t *song, enum meta_type_e which ) {
         return;
     }
 
-    metaname = build_fullfilename( song, db_extensions[which].which_basename );
+    /* If we haven't been loaded before and this is TYPE_META, set meta_key_count to 0 */
+    if( which == TYPE_META && song->meta_key_count == -1 ) {
+        song->meta_key_count = 0;
+    }
+
+    metaname = NULL;
+    /* If this is an empeg, we may be reading the original empeg's player's
+     * database files.  In which case music files we find that end it a 0,
+     * will probably have a meta file that ends instead with 1.  We will
+     * load that instead of the normal .info file. */
+#ifdef EMPEG
+    if( which == TYPE_META ) {
+        int length;
+        length = strlen(song->filename);
+        if( length > 0 && song->filename[length-1] == '0' ) {
+            song->filename[length-1] = '1';
+            metaname = build_fullfilename( song, BASENAME_SONG );
+            song->filename[length-1] = '0';
+        }
+    }
+#endif
+    if( metaname == NULL ) {
+        metaname = build_fullfilename( song, db_extensions[which].which_basename );
+    }
 
     /* Parse the file unless we want to reload/save from the original song file */
     if( config.db_overwriteinfo && config.db_saveinfo && which == TYPE_META ) {
-        success = false;
+        success = FALSE;
     } else {
         success = parse_file( metaname, db_extensions[which].add_data, (void *)song );
     }
@@ -403,27 +452,38 @@ void load_meta_data( song_info_t *song, enum meta_type_e which ) {
     /* If we didn't load from the stat/info file, and we have a info file,
      * try to load it from the original song file */
     if( !success && which == TYPE_META ) {
-        song_type = get_song_type( song->filename );
+        /* perhaps we haven't figured out what type of song it is yet */
+        if( song->song_type == -1 ) {
+            song->song_type = get_song_type( song->basename[BASENAME_SONG], song->filename );
+        }
 
-        if( song_type != TYPE_UNKNOWN ) {
+        if( song->song_type != TYPE_UNKNOWN ) {
             filename = build_fullfilename( song, BASENAME_SONG );
 
-            if( song_type == TYPE_OGG ) {
-                ogg_load_meta( (void *)song, filename );
-            } else {
-                /* This routine does nothing right now */
-                mp3_load_meta( (void *)song, filename );
+            switch( song->song_type ) {
+                case TYPE_OGG:
+                    ogg_load_meta( (void *)song, filename );
+                    break;
+                case TYPE_MP3:
+                    mp3_load_meta( (void *)song, filename );
+                    break;
+                case TYPE_FLAC:
+                    flac_load_meta( (void *)song, filename );
+                    break;
+                case TYPE_UNKNOWN:
+                    /* can't get here */
+                    break;
             }
             squash_free( filename );
 
             if( config.db_saveinfo ) {
 #ifdef NO_VORBIS_COMMENT
-                if( song_type != TYPE_OGG ) {
+                if( song->song_type != TYPE_OGG ) {
 #else
                 if( TRUE ) {
 #endif
 #ifdef NO_ID3LIB
-                if( song_type != TYPE_MP3 ) {
+                if( song->song_type != TYPE_MP3 ) {
 #else
                 if( TRUE ) {
 #endif
@@ -460,23 +520,42 @@ void load_meta_data( song_info_t *song, enum meta_type_e which ) {
  */
 void load_all_meta_data( enum meta_type_e which ) {
     int i;
+    squash_log("loading %d (write lock for database will oscillate)", which);
+#ifdef EMPEG
+    squash_rlock( database_info.lock );
+#else
+    squash_wlock( database_info.lock );
+#endif
     for( i = 0; i < database_info.song_count; i++ ) {
+#ifdef EMPEG
+        squash_runlock( database_info.lock );
         squash_wlock( database_info.lock );
-        /* If the type is meta, we need to be careful */
-        if( which == TYPE_META ) {
-            /* if we haven't loaded before, set the key count to 0,
-             * otherwise, skip loading this song (since it has already
-             * been loaded */
-            if( database_info.songs[i].meta_key_count == -1 ) {
-                database_info.songs[i].meta_key_count = 0;
-            } else {
-                squash_wunlock( database_info.lock );
-                continue;
-            }
+#endif
+        /* If the type is meta, we need to be careful, and not load this song a second time  */
+        if( which == TYPE_META && database_info.songs[i].meta_key_count != -1 ) {
+            squash_wunlock( database_info.lock );
+            continue;
         }
         load_meta_data( &database_info.songs[i], which );
+        if( i % 500 == 0 ) {
+            squash_log("%d of %d load complete", i, database_info.song_count);
+        }
+#ifdef EMPEG
         squash_wunlock( database_info.lock );
+        sched_yield();
+        squash_rlock( database_info.lock );
+#endif
     }
+
+    /* go back and save any changes that may have happened while we were loading */
+    for( i = 0; i < database_info.song_count; i++ ) {
+        save_song( &database_info.songs[i] );
+    }
+#ifdef EMPEG
+    squash_runlock( database_info.lock );
+#else
+    squash_wunlock( database_info.lock );
+#endif
 }
 
 
@@ -552,11 +631,15 @@ void set_stat_data( void *data, char *header, char *key, char *value ) {
 
     /* Add it to the structure */
     if( strncasecmp("play_count", key, 11) == 0 ) {
-        song->stat.play_count = int_value;
+        song->stat.play_count += int_value;
     } else if( strncasecmp("repeat_counter", key, 15) == 0 ) {
         song->stat.repeat_counter = int_value;
     } else if( strncasecmp("skip_count", key, 11) == 0 ) {
-        song->stat.skip_count = int_value;
+        song->stat.skip_count += int_value;
+    } else if( strncasecmp("manual_rating", key, 14) == 0 ) {
+        if( song->stat.manual_rating == -1 ) {
+            song->stat.manual_rating = int_value;
+        }
     }
 
     squash_free( key );
@@ -566,8 +649,8 @@ void set_stat_data( void *data, char *header, char *key, char *value ) {
 /*
  * Load a song into the database.  This is called from _walk_filesystem()
  */
-void _load_file( char *filename ) {
-    enum song_type_e type;
+void _load_file( char *filename, bool trust ) {
+    enum song_type_e type = TYPE_UNKNOWN;
     song_info_t *song;
 
     /* Avoid mistakes */
@@ -575,11 +658,13 @@ void _load_file( char *filename ) {
         return;
     }
 
-    /* Determine the song type */
-    type = get_song_type( filename );
+    /* Determine the song type unless we trust this file */
+    if( !trust ) {
+        type = get_song_type( config.db_paths[ BASENAME_SONG ], filename );
+    }
 
-    /* If filename is a known music type, load it into the database */
-    if( type != TYPE_UNKNOWN ) {
+    /* If filename is a known music type or we trust it, load it into the database */
+    if( type != TYPE_UNKNOWN || trust ) {
         /* Make sure we have enough memory */
         squash_ensure_alloc( database_info.song_count, database_info.song_count_allocated,
                 database_info.songs, sizeof(song_info_t), INITIAL_DB_SIZE, *=2 );
@@ -592,22 +677,45 @@ void _load_file( char *filename ) {
         song->basename[ BASENAME_STAT ] = config.db_paths[ BASENAME_STAT ];
         song->meta_keys = NULL;
         song->meta_key_count = -1;
+        song->stat.play_count = 0;
+        song->stat.skip_count = 0;
+        song->stat.repeat_counter = 0;
+        song->stat.manual_rating = -1;
         song->stat.changed = FALSE;
+        song->play_length = -1;
+        song->song_type = -1;
 
         /* Update the counter */
         database_info.song_count++;
 
+#ifdef EMPEG
+        if( database_info.song_count % 50 == 0 ) {
+            int x = database_info.song_count, i = 3;
+            char buf[5] = "0000";
+            while( i >= 0 && x > 0 ) {
+                buf[i] += x % 10;
+                x /= 10;
+                i--;
+            }
+            vfdlib_clear( display_info.screen, 0 );
+            vfdlib_drawText( display_info.screen, "Squash Loading, version " SQUASH_VERSION, 0, 0, 2, 3);
+            vfdlib_drawText( display_info.screen, buf, 0, 6, 2, 3 );
+            ioctl(display_info.screen_fd, _IO('d', 0));
+        }
+#endif
+#ifndef NO_NCURSES
         /* Display progress every 500 songs */
         if( database_info.song_count % 500 == 0) {
             draw_info();
         }
+#endif
     }
 }
 
 /*
  * Walk a filesystem and call loader() on any files found
  */
-void _walk_filesystem( char *base_dir, void(*loader)(char *) ) {
+void _walk_filesystem( char *base_dir, void(*loader)(char *, bool) ) {
     DIR *dir;
     struct dirent *file_entry;
     struct stat file_stat;
@@ -669,7 +777,7 @@ void _walk_filesystem( char *base_dir, void(*loader)(char *) ) {
             if( S_ISDIR(file_stat.st_mode) ) {
                 _walk_filesystem( file_path, loader );
             } else {
-                loader( file_path );
+                loader( file_path, FALSE );
             }
         }
 
