@@ -39,7 +39,8 @@ void *mp3_open( char *filename, sound_format_t *sound_format ) {
 
     /* Open the song */
     if( (mp3_data->file = fopen(filename, "r")) == NULL ) {
-        squash_error( "Unable to open file" );
+        return (void *)NULL;
+        // squash_error( "Unable to open file" );
     }
 
     /* Open the mp3 file */
@@ -47,7 +48,8 @@ void *mp3_open( char *filename, sound_format_t *sound_format ) {
     mp3_data->file_size = file_stat.st_size;
     mp3_data->buffer = mmap(0, mp3_data->file_size, PROT_READ, MAP_SHARED, fileno(mp3_data->file), 0);
     if( mp3_data->buffer == (void *)-1 ) {
-        squash_error( "Unable to open MP3 file" );
+        return (void *)NULL;
+        // squash_error( "Unable to open MP3 file" );
     }
 
     mp3_data->pcm_data = NULL;
@@ -70,9 +72,9 @@ void *mp3_open( char *filename, sound_format_t *sound_format ) {
         break;
     }
 
-    sound_format->channels = MAD_NCHANNELS(&m_header);
     sound_format->rate = m_header.samplerate;
-    sound_format->byte_format = AO_FMT_LITTLE;
+    sound_format->channels = MAD_NCHANNELS(&m_header);
+    sound_format->byte_format = SOUND_LITTLE;
     sound_format->bits = 16;
 
     mad_header_finish(&m_header);
@@ -263,9 +265,6 @@ frame_data_t mp3_decode_frame( void *data ) {
     }
     frame_data.pcm_size = pcm_size;
     frame_data.position = mad_timer_count(mp3_data->timer, MAD_UNITS_MILLISECONDS);
-    if( frame_data.position > mp3_data->duration ) {
-        frame_data.position = mp3_data->duration;
-    }
 
     /* Setup PCM buffer */
     if( frame_data.pcm_size <= 0 ) {
@@ -282,9 +281,17 @@ frame_data_t mp3_decode_frame( void *data ) {
  * Seek to the seek_time position (in milliseconds) in
  * the opened song
  */
-void mp3_seek( void *data, long seek_time ) {
+void mp3_seek( void *data, long seek_time, long duration ) {
     mp3_data_t *mp3_data = (mp3_data_t *)data;
-    long new_file_position = (long)((double)mp3_data->file_size * seek_time / mp3_data->duration);
+    long new_file_position;
+    if( duration <= 0 && seek_time != 0 ) {
+        return;
+    }
+    if( seek_time == 0 ) {
+        new_file_position = 0;
+    } else {
+        new_file_position = (long)((double)mp3_data->file_size * seek_time / duration);
+    }
     mad_stream_buffer(&mp3_data->stream, (unsigned char *)&mp3_data->buffer[new_file_position], mp3_data->file_size - new_file_position);
     mad_timer_set( &mp3_data->timer, 0, seek_time, 1000 );
     mad_frame_mute( &mp3_data->frame );
@@ -339,6 +346,7 @@ unsigned int mad_to_16bit(mad_fixed_t f) {
  */
 long mp3_calc_duration(void * data) {
     mp3_data_t *mp3_data = (mp3_data_t *)data;
+    long duration;
     struct mad_header m_header;
 
     while(1) {
@@ -354,7 +362,7 @@ long mp3_calc_duration(void * data) {
         mad_timer_add(&mp3_data->timer, m_header.duration);
     }
 
-    mp3_data->duration = mad_timer_count(mp3_data->timer, MAD_UNITS_MILLISECONDS);
+    duration = mad_timer_count(mp3_data->timer, MAD_UNITS_MILLISECONDS);
 
     mad_header_finish(&m_header);
     mad_stream_finish(&mp3_data->stream);
@@ -363,5 +371,5 @@ long mp3_calc_duration(void * data) {
     mad_timer_reset(&mp3_data->timer);
     mad_stream_buffer(&mp3_data->stream, (unsigned char *)mp3_data->buffer, mp3_data->file_size);
 
-    return mp3_data->duration;
+    return duration;
 }
